"""QubitBoost SDK integration layer.

Optional bridge between qb-compiler (open source) and the
QubitBoost SDK (proprietary).  qb-compiler works standalone
without the SDK installed.
"""

from __future__ import annotations
